{------------------------------------------------------------------------------}
{
   Unit Name: tcpipform
   Purpose  : To give TCP/IP connections between machines
   Author   : Vesa Lappalainen
   Date     : 20.11.1999
   Changed  :

   ToDo     :
    1) cr:n ksittelyn voisi siirt suoraan TCP/IP-komponenttiin,
       eli haluttaessa voisi automaattisesti list lhetyksiin cr
       ja jos tulee rivi, jossa on cr tekstin sisll, tmn voisi
       antaa komponentin kyttjlle useana palana
    2) voisi tehd oikean raahattavan komponentin, joka itse loisi
       tmn formin   

   Tarkoitus on tarjota komponentti, joka voi toimia samalla sek
   palvelimena, ett asiakkaana.

   Jos asiakas saa merkkijonon, se vlitetn kaikille palveluun
   liittyneille (jos niit on).  Samoin jos palvelin saa merkkijonon,
   vlitt se merkkijonon edelleen kaikille palveluun liittyneille
   (paitsi lhettjlle) ja mys sille palvelimelle, jonka asiakkaana
   ollaan (jos ollaan).
   Tuli merkkijono kenelt tahansa, se vlitetn mys tiedoksi tmn
   komponentin kyttjlle.

   Tm komponentti tarjoaa mys SendInformation-metodin, mill komponentin
   kyttj voi lhett merkkijonon kaikille asiakkaille ja sille
   palvelimelle, johon komponentti on liittynyt asiakkaaksi.
}
{------------------------------------------------------------------------------}

unit tcpipform;

interface

uses
  SysUtils, Classes,
{$ifdef CLX}
  QGraphics, QControls, QForms, QDialogs, QStdCtrls, QExtCtrls,QComCtrls,
{$else}
  Windows, Messages,
  Graphics, Controls, Forms, Dialogs, StdCtrls,  ExtCtrls,ComCtrls,
{$endif}
  kinicomp, IdTCPConnection, IdTCPClient, IdBaseComponent, IdComponent,
  IdTCPServer, IdAntiFreezeBase, IdAntiFreeze;

type
  TTCPIPGetInformationEvent = function (const s:string):boolean of object;

  TFormTCPIP = class;

  TClientThread = class(TThread)
  private
    form:TFormTCPIP;
    data :string;
    procedure Send;
  public
    constructor Create(form:TFormTCPIP); virtual;
    procedure Execute; override;
  end;


  TFormTCPIP = class(TForm)
    PanelServer: TPanel;
    PanelClient: TPanel;
    GroupBoxServerSettings: TGroupBox;
    GroupBoxClientSettings: TGroupBox;
    MemoClientError: TMemo;
    MemoServerError: TMemo;
    PanelServerPort: TPanel;
    EditServerPort: TEditIni;
    PanelClientPort: TPanel;
    EditClientPort: TEditIni;
    PanelHost: TPanel;
    EditHost: TEditIni;
    NotebookServer: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    ButtonServerListen: TButton;
    ButtonClose: TButton;
    LabelConnectionsText: TLabel;
    LabelConnections: TLabel;
    NotebookClient: TPageControl;
    TabSheet3: TTabSheet;
    TabSheet4: TTabSheet;
    ButtonClientConnect: TButton;
    ButtonClientClose: TButton;
    Server: TIdTCPServer;
    Client: TIdTCPClient;
    procedure ButtonServerListenClick(Sender: TObject);
    procedure ButtonClientConnectClick(Sender: TObject);
    procedure ButtonCloseClick(Sender: TObject);
    procedure ButtonClientCloseClick(Sender: TObject);
    procedure ClientConnected(Sender: TObject);
    procedure ClientDisconnected(Sender: TObject);
    procedure ServerConnect(AThread: TIdPeerThread);
    procedure ServerDisconnect(AThread: TIdPeerThread);
    procedure ServerExecute(AThread: TIdPeerThread);
    procedure ClientStatus(axSender: TObject; const axStatus: TIdStatus;
      const asStatusText: String);
    procedure FormDestroy(Sender: TObject);
  private
    thread : TClientThread;
    FOnGetInformation: TTCPIPGetInformationEvent;
    procedure UpdateConnections(i:integer=0); virtual;
  public
    procedure SendInformation(const s:string;WhoNot: TIdPeerThread=nil;alsoClient:boolean=true); virtual;
  published
    property OnGetInformation : TTCPIPGetInformationEvent read FOnGetInformation write FOnGetInformation;
  end;

var
  FormTCPIP: TFormTCPIP;

implementation

{$ifdef CLX}
{R *.xfm}
{$else}
{$R *.dfm}
{$endif}


const pgConnected = 1;
      pgDisConnected = 0;

{ TFormTCPIP }

procedure TFormTCPIP.SendInformation(const s:string;WhoNot: TIdPeerThread;alsoClient:boolean);
var i:integer; list : TList; t: TIdPeerThread;
begin
  if ( Client.Connected ) and ( alsoClient ) then Client.Write(s);
  try
    List := Server.Threads.LockList;
    for i:=0 to List.Count-1 do begin
      t := List[i];
      if ( t = WhoNot ) then continue;
      t.Connection.Write(s);
    end;
  finally
    Server.Threads.UnLockList;
  end;
end;

procedure TFormTCPIP.ButtonServerListenClick(Sender: TObject);
begin
  Server.Active := false;
  Server.DefaultPort := StrToIntDef(EditServerPort.Text,1024);
  Server.Active := true;
  NotebookServer.ActivePageIndex := pgConnected;
  UpdateConnections;
end;

procedure TFormTCPIP.ButtonClientConnectClick(Sender: TObject);
begin
  Client.Disconnect;
  Client.Port := StrToIntDef(EditClientPort.Text,0);
  Client.Host := EditHost.Text;
  Client.Connect;
end;

procedure TFormTCPIP.ButtonCloseClick(Sender: TObject);
begin
  NotebookServer.ActivePageIndex := pgDisConnected;
  Server.Active := false;
  UpdateConnections;
end;

procedure TFormTCPIP.ButtonClientCloseClick(Sender: TObject);
begin
  NoteBookClient.ActivePageIndex := pgDisConnected;
  Client.Disconnect;
end;


procedure TFormTCPIP.UpdateConnections(i:integer);
var n:integer; list : TList;
begin
  try
    n := 0;
    List := Server.Threads.LockList;
    n:=List.Count;
  finally
    Server.Threads.UnLockList;
  end;
  LabelConnections.Caption := IntToStr(n+i);
end;

procedure TFormTCPIP.ClientConnected(Sender: TObject);
begin
  NoteBookClient.ActivePageIndex := pgConnected;
  thread := TClientThread.Create(self);
end;

procedure TFormTCPIP.ClientDisconnected(Sender: TObject);
begin
  NoteBookClient.ActivePageIndex := pgDisConnected;
  if ( Assigned(thread) ) then thread.DoTerminate;
  thread := nil;
end;

procedure TFormTCPIP.FormDestroy(Sender: TObject);
begin
  if ( Assigned(thread) ) then thread.DoTerminate;
end;

procedure TFormTCPIP.ServerConnect(AThread: TIdPeerThread);
begin
  UpdateConnections;
end;

procedure TFormTCPIP.ServerDisconnect(AThread: TIdPeerThread);
begin
  UpdateConnections(-1);
end;

procedure TFormTCPIP.ServerExecute(AThread: TIdPeerThread);
var s:string;
begin
  s := AThread.Connection.CurrentReadBuffer;
  if ( s = '' ) then exit;
  if ( Assigned(OnGetInformation) ) then OnGetInformation(s);
  SendInformation(s,AThread,true);
end;

procedure TFormTCPIP.ClientStatus(axSender: TObject;
  const axStatus: TIdStatus; const asStatusText: String);
var s:string;
begin
  s := asStatusText;
  if ( s = '' ) then exit;
  MemoClientError.Lines.Add(s);
  if axStatus = hsDisconnecting then begin
    ClientDisconnected(nil);
    exit;
  end;
end;

{ TClientThread }

constructor TClientThread.Create(form:TFormTCPIP);
begin
  inherited Create(false);
  self.FreeOnTerminate := true;
  self.form := form;
end;

procedure TClientThread.Send;
begin
  if ( Assigned(form.OnGetInformation) ) then form.OnGetInformation(data);
  form.SendInformation(data,nil,false);
end;

procedure TClientThread.Execute;
var s:string;
begin
  while ( form.Client.Connected ) and ( not Terminated ) do begin
    s := form.Client.CurrentReadBuffer;
    if ( s = '' ) then continue;
    data := s;
    Synchronize(Send);
  end;
end;

end.
