{------------------------------------------------------------------------------}
{
   Unit Name: tcpipform
   Purpose  : To give TCP/IP connections between machines
   Author   : Vesa Lappalainen
   Date     : 20.11.1999
   Changed  : 14.10.2001
    - muutettu kyttmn Indy-komponentteja
        IdTCPClient
        IdTCPServer
      entisten ServerSocket ja ClientSocket tilalla (tytyy tunnustaa
      ett Socket-komponentit olivta monin verroihn helpompia kytt).
      Nin saatu toimimaan sek VCL ett CLX.

   ToDo     :
    1) cr:n ksittelyn voisi siirt suoraan TCP/IP-komponenttiin,
       eli haluttaessa voisi automaattisesti list lhetyksiin cr
       ja jos tulee rivi, jossa on cr tekstin sisll, tmn voisi
       antaa komponentin kyttjlle useana palana
    2) voisi tehd oikean raahattavan komponentin, joka itse loisi
       tmn formin

   Tarkoitus on tarjota komponentti, joka voi toimia samalla sek
   palvelimena, ett asiakkaana.

   Jos asiakas saa merkkijonon, se vlitetn kaikille palveluun
   liittyneille (jos niit on).  Samoin jos palvelin saa merkkijonon,
   vlitt se merkkijonon edelleen kaikille palveluun liittyneille
   (paitsi lhettjlle) ja mys sille palvelimelle, jonka asiakkaana
   ollaan (jos ollaan).
   Tuli merkkijono kenelt tahansa, se vlitetn mys tiedoksi tmn
   komponentin kyttjlle.

   Tm komponentti tarjoaa mys SendInformation-metodin, mill komponentin
   kyttj voi lhett merkkijonon kaikille asiakkaille ja sille
   palvelimelle, johon komponentti on liittynyt asiakkaaksi.

   On mys metodit Listen palvelimen kynnistmiseksi ja Connect
   asiakkaaksi liittymiseksi.

   Vastaavasti CloseServer ja CloseClient.

   Yksinkertaisimmillaan tm lomake liitetn projektiin ja
   jossakin kohti koodia nytetn lomake, jotta asetuksia psee
   vaihtamaan.  Jos valmiit asetukset kelpaavat, ttkn ei tarvitse tehd
   (ajetaan jossakin tcpipform.Listen tai tcpipform.Connect).
   Mieluummin tietysti lomake luodaan dynaamisesti.

   Sitten ksitelln tapahtuma OnGetInformation.
   Katso esimerkki tcpiptest.pas.
}
{------------------------------------------------------------------------------}

unit tcpipformCS;

interface

uses
  SysUtils,
  Classes,
{$ifdef CLX}
  QGraphics, QControls, QForms, QDialogs, QStdCtrls, QExtCtrls,QComCtrls,
{$else}
  Windows, Messages,
  Graphics, Controls, Forms, Dialogs, StdCtrls,  ExtCtrls,ComCtrls,
{$endif}
  IdTCPConnection, IdTCPClient, IdBaseComponent, IdComponent,
  IdTCPServer, kEditPnl, Sockets;


type
  TTCPIPGetInformationEvent = function (const s:string):boolean of object;

  TFormTCPIP = class;



  TFormTCPIP = class(TForm)
    PanelServer: TPanel;
    PanelClient: TPanel;
    GroupBoxServerSettings: TGroupBox;
    GroupBoxClientSettings: TGroupBox;
    MemoClientError: TMemo;
    MemoServerError: TMemo;
    NotebookServer: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    ButtonServerListen: TButton;
    ButtonClose: TButton;
    LabelConnectionsText: TLabel;
    LabelConnections: TLabel;
    NotebookClient: TPageControl;
    TabSheet3: TTabSheet;
    TabSheet4: TTabSheet;
    ButtonClientConnect: TButton;
    ButtonClientClose: TButton;
    EditServerPort: TEditPanel;
    EditServerPortEdit: TEdit;
    EditHost: TEditPanel;
    EditHostEdit: TEdit;
    EditClientPort: TEditPanel;
    EditClientPortEdit: TEdit;
    Server: TIdTCPServer;
    Client: TTcpClient;
    procedure ButtonServerListenClick(Sender: TObject);
    procedure ButtonClientConnectClick(Sender: TObject);
    procedure ButtonCloseClick(Sender: TObject);
    procedure ButtonClientCloseClick(Sender: TObject);
    procedure ServerConnect(AThread: TIdPeerThread);
    procedure ServerDisconnect(AThread: TIdPeerThread);
    procedure ServerExecute(AThread: TIdPeerThread);
    procedure ServerStatus(axSender: TObject; const axStatus: TIdStatus;
      const asStatusText: String);
    procedure ClientError(Sender: TObject; SocketError: Integer);
    procedure ClientConnect(Sender: TObject);
    procedure ClientDisconnect(Sender: TObject);
    procedure ClientReceive(Sender: TObject; Buf: PChar;
      var DataLen: Integer);
  private
    FOnGetInformation: TTCPIPGetInformationEvent;
    procedure UpdateConnections(i:integer=0); virtual;
    procedure ClientThreadTermiante(Sender: TObject);
  public
    procedure SendInformation(const s:string;WhoNot: TIdPeerThread=nil;alsoClient:boolean=true); virtual;
    procedure Connect; virtual;
    procedure Listen; virtual;
    procedure CloseServer; virtual;
    procedure CloseClient; virtual;
  published
    property OnGetInformation : TTCPIPGetInformationEvent read FOnGetInformation write FOnGetInformation;
  end;

var
  FormTCPIP: TFormTCPIP;

implementation

{$ifdef CLX}
{$R *.xfm}
{$else}
{$R *.dfm}
{$endif}


const pgConnected = 1;
      pgDisConnected = 0;

{ TFormTCPIP }

procedure TFormTCPIP.SendInformation(const s:string;WhoNot: TIdPeerThread;alsoClient:boolean);
var i:integer; list : TList; t: TIdPeerThread;
begin
  if ( Client.Connected ) and ( alsoClient ) then Client.Sendln(s+#10);
  try
    List := Server.Threads.LockList;
    for i:=0 to List.Count-1 do begin
      t := List[i];
      if ( t = WhoNot ) then continue;
      t.Connection.Write(s);
    end;
  finally
    Server.Threads.UnLockList;
  end;
end;

procedure TFormTCPIP.Listen;
begin
  Server.Active := false;
  Server.DefaultPort := StrToIntDef(EditServerPort.Text,1024);
  Server.Active := true;
  NotebookServer.ActivePageIndex := pgConnected;
  UpdateConnections;
end;

procedure TFormTCPIP.ButtonServerListenClick(Sender: TObject);
begin
  Listen;
end;

procedure TFormTCPIP.Connect;
begin
  Client.Disconnect;
  Client.RemotePort := EditClientPort.Text;
  Client.RemoteHost := EditHost.Text;
  Client.Connect;
end;

procedure TFormTCPIP.ButtonClientConnectClick(Sender: TObject);
begin
  Connect;
end;

procedure TFormTCPIP.CloseServer;
begin
  NotebookServer.ActivePageIndex := pgDisConnected;
  Server.Active := false;
  UpdateConnections;
end;

procedure TFormTCPIP.ButtonCloseClick(Sender: TObject);
begin
  CloseServer;
end;

procedure TFormTCPIP.CloseClient;
begin
  NoteBookClient.ActivePageIndex := pgDisConnected;
  Client.Disconnect;
end;


procedure TFormTCPIP.ButtonClientCloseClick(Sender: TObject);
begin
  CloseClient;
end;

procedure TFormTCPIP.UpdateConnections(i:integer);
var n:integer; list : TList;
begin
  try
    n := 0;
    List := Server.Threads.LockList;
    n:=List.Count;
  finally
    Server.Threads.UnLockList;
  end;
  LabelConnections.Caption := IntToStr(n+i);
end;

procedure TFormTCPIP.ClientThreadTermiante(Sender: TObject);
begin
end;

procedure TFormTCPIP.ServerConnect(AThread: TIdPeerThread);
begin
  UpdateConnections;
end;

procedure TFormTCPIP.ServerDisconnect(AThread: TIdPeerThread);
begin
  UpdateConnections(-1);
end;

procedure TFormTCPIP.ServerExecute(AThread: TIdPeerThread);
var s:string;
begin
  s := AThread.Connection.CurrentReadBuffer;
  if ( s = '' ) then exit;
  if ( Assigned(OnGetInformation) ) then OnGetInformation(s);
  SendInformation(s,AThread,true);
end;

procedure TFormTCPIP.ServerStatus(axSender: TObject;
  const axStatus: TIdStatus; const asStatusText: String);
var s:string;
begin
  s := asStatusText;
  if ( s = '' ) then exit;
  MemoClientError.Lines.Add(s);
end;

procedure TFormTCPIP.ClientError(Sender: TObject; SocketError: Integer);
var s:string;
begin
  s := IntToStr(SocketError);
  if ( s = '' ) then exit;
  MemoClientError.Lines.Add(s);
end;

procedure TFormTCPIP.ClientConnect(Sender: TObject);
begin
  NoteBookClient.ActivePageIndex := pgConnected;
end;

procedure TFormTCPIP.ClientDisconnect(Sender: TObject);
begin
  NoteBookClient.ActivePageIndex := pgDisConnected;
end;

procedure TFormTCPIP.ClientReceive(Sender: TObject; Buf: PChar;
  var DataLen: Integer);
var data:string;
begin
  data := Buf;
  SendInformation(data);
end;

end.
